package
{
	import flash.text.*;
	import flash.display.*;
	import flash.events.Event;
	import flash.geom.Rectangle;
	
	import flashx.textLayout.formats.TextAlign;
	
	import ru.kozlovskij.air.mobile.iOS.text.softKeyboard.SoftKeyboardDetector;
	import ru.kozlovskij.air.mobile.iOS.text.softKeyboard.SoftKeyboardDetectorEvent;
	
	
	/**
	 * @author Aleksandr Kozlovskij (created: Oct 26, 2011)
	 */
	public class iOSSoftKeyboardControllersExample extends Sprite
	{
		private const _log:TextField = new TextField();
		private var detector:SoftKeyboardDetector;
		
		//------------ constructor ------------//
		
		public function iOSSoftKeyboardControllersExample()
		{
			super();
			
			// support autoOrients
			stage.align = StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			
			stage ? initialize() : addEventListener(Event.ADDED_TO_STAGE, initialize);
		}
		
		//------------ initialize ------------//
		
		private function initialize(e:Event = null):void
		{
			e && removeEventListener(Event.ADDED_TO_STAGE, initialize);
			
			// support autoOrients:
			stage.align = StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.addEventListener(Event.RESIZE, stageResizeHandler);
			
			// create detector:
			{
				detector = new SoftKeyboardDetector(stage);
				// set the time for waiting completion of native keyboard animation.
				// Recommended values range: from 400 to 800.
				// Note: The higher the value - the more stable operation of the analyzer.
				detector.keyboardAnimationWaitingTime = 600;
				
				detector.addEventListener(SoftKeyboardDetectorEvent.ACTIVATE_FLAT, logHandler);
				detector.addEventListener(SoftKeyboardDetectorEvent.ACTIVATE_SPLIT, logHandler);
				detector.addEventListener(SoftKeyboardDetectorEvent.CHANGE_FLAT_TO_SPLIT, logHandler);
				detector.addEventListener(SoftKeyboardDetectorEvent.CHANGE_SPLIT_TO_FLAT, logHandler);
				detector.addEventListener(SoftKeyboardDetectorEvent.DEACTIVATE_SOFTKEYBOARD, logHandler);
				detector.addEventListener(SoftKeyboardDetectorEvent.RELATED_OBJECT_CHANGE, logHandler);
			}
			
			// create UI:
			configureAndAddLoggingTF();
			createAndAddStageTextLines();
			stageResizeHandler();
		}
		
		private function configureAndAddLoggingTF():void
		{
			_log.selectable = false;
			_log.multiline = true;
			_log.wordWrap = true;
			_log.mouseEnabled = true;
			_log.alwaysShowSelection = false;
			_log.type = TextFieldType.DYNAMIC;
			_log.antiAliasType = AntiAliasType.ADVANCED;
			_log.mouseWheelEnabled = true;
			_log.needsSoftKeyboard = false;
			_log.border = true;
			_log.borderColor = 0x0;
			_log.width = stage.stageWidth - 1;
			_log.height = Math.min(stage.stageWidth, stage.stageHeight) / 4;
			_log.addEventListener(Event.CHANGE, scrollMax);
			addChild(_log);
		}
		
		private function createAndAddStageTextLines():void
		{
			var st:StageText;
			var vp:Rectangle;
			var i:int;
			
			// create singleline fields:
			for(i; i < 4; ++i)
			{
				st = createStageText();
				vp = new Rectangle(0, _log.y + _log.height + 22 * i, stage.stageWidth / 2, 22);
				detector.listenStageText(st);
				st.stage = stage;
				st.viewPort = vp;
				//st.assignFocus();
			}
			
			// create multiline field:
			{
				st = createStageText(i);
				detector.listenStageText(st);
				vp.y += vp.height;
				vp.height += 22 * i;
				//vp = new Rectangle(stage.stageWidth / 2, _log.y + _log.height, stage.stageWidth / 2, 22 * i);
				st.stage = stage;
				st.viewPort = vp;
				//st.assignFocus();
			}
		}
		
		private function createStageText(linesNum:uint = 1):StageText
		{
			linesNum < 1 && (linesNum = 1);
			
			const options:StageTextInitOptions = new StageTextInitOptions(linesNum > 1);
			const result:StageText = new StageText(options);
			
			result.returnKeyLabel = ReturnKeyLabel.NEXT;
			result.softKeyboardType = SoftKeyboardType.CONTACT;
			result.autoCorrect = true;
			result.fontSize = 19;
			result.fontFamily = "Courier New";
			result.textAlign = TextAlign.LEFT;
			
			if(options.multiline)
			{
				var text:String = '';
				while(linesNum--)
					text += "multiline line: " + linesNum + "\n";
				result.text = text.substr(0, text.length - 1);
			}
			else
				result.text = "monoline StageText";
			
			return result;
		}
		
		//--------------- ctrl ---------------//
		
		private function log(value:String):void
		{
			_log.appendText(value + '\n');
			scrollMax();
		}
		
		private function scrollMax(e:Event = null):void
		{
			//_log.setSelection(_log.length, _log.length);
			_log.scrollV = _log.maxScrollV;
		}
		
		//------------ get / set -------------//
		
		//------- handlers / callbacks -------//
		
		private function stageResizeHandler(e:Event = null):void
		{
			_log.width = stage.stageWidth - 1;
			_log.height = Math.min(stage.stageWidth, stage.stageHeight) / 4;;
		}
		
		private function logHandler(e:Event):void
		{
			log(e.type + ' : ' + e.target);
			
			if(e.target == detector)
				log('view type: "' + detector.viewTypeString.toUpperCase() + '"');
			
			scrollMax();
		}
	}
}